@@ -4,6 +4,7 @@ Cross Site Request Forgery Middleware.
 This module provides a middleware that implements protection
 against request forgeries from other sites.
 """
+
 import logging
 import string
 from collections import defaultdict
@@ -41,7 +42,6 @@ REASON_INVALID_CHARACTERS = "has invalid characters"
 CSRF_SECRET_LENGTH = 32
 CSRF_TOKEN_LENGTH = 2 * CSRF_SECRET_LENGTH
 CSRF_ALLOWED_CHARS = string.ascii_letters + string.digits
-CSRF_SESSION_KEY = "_csrftoken"
 
 
 def _get_new_csrf_string():
@@ -179,6 +179,9 @@ class CsrfViewMiddleware:
                 # saved anyways.
                 request.META["CSRF_COOKIE"] = csrf_secret
 
+        if csrf_response := self._get_csrf_response(request):
+            return csrf_response
+
         response = self.get_response(request)
 
         if request.META.get("CSRF_COOKIE_NEEDS_UPDATE"):
@@ -394,23 +397,16 @@ class CsrfViewMiddleware:
             reason = self._bad_token_message("incorrect", token_source)
             raise RejectRequest(reason)
 
-    def process_view(self, request, callback, callback_args, callback_kwargs):
+    def _get_csrf_response(self, request):
         # Wait until request.META["CSRF_COOKIE"] has been manipulated before
         # bailing out, so that get_token still works
-        if getattr(callback, "csrf_exempt", False):
+        if getattr(request, "csrf_exempt", True):
             return None
 
         # Assume that anything not defined as 'safe' by RFC 9110 needs protection
         if request.method in ("GET", "HEAD", "OPTIONS", "TRACE"):
             return None
 
-        if getattr(request, "_dont_enforce_csrf_checks", False):
-            # Mechanism to turn off CSRF checks for test suite. It comes after
-            # the creation of CSRF cookies, so that everything else continues
-            # to work exactly the same (e.g. cookies are sent, etc.), but
-            # before any branches that call the _reject method.
-            return None
-
         # Reject the request if the Origin header doesn't match an allowed
         # value.
         if "HTTP_ORIGIN" in request.META:
