@@ -6,20 +6,23 @@ from plain.http import Response
 from plain.views.exceptions import ResponseException
 
 
-def dd(obj):
+def dd(*objs):
     """
     Dump and die.
 
     Dump the object and raise a ResponseException with the dump as the response content.
     """
-    dump_str = pformat(obj)
 
-    print(f"Dumping object:\n{dump_str}")
+    print(f"Dumping objects:\n{'\n'.join([pformat(obj) for obj in objs])}")
+
+    dump_strs = [
+        Markup("<pre><code>") + escape(pformat(obj)) + Markup("</code></pre>")
+        for obj in objs
+    ]
+    combined_dump_str = Markup("\n\n").join(dump_strs)
 
     response = Response()
     response.status_code = 500
-    response.content = (
-        Markup("<pre><code>") + escape(dump_str) + Markup("</code></pre>")
-    )
+    response.content = combined_dump_str
     response.content_type = "text/html"
     raise ResponseException(response)
