@@ -4,8 +4,7 @@ import types
 from plain.exceptions import ImproperlyConfigured
 from plain.logs import log_response
 from plain.runtime import settings
-from plain.signals import request_finished
-from plain.urls import get_resolver, set_urlconf
+from plain.urls import get_resolver
 from plain.utils.module_loading import import_string
 
 from .exception import convert_exception_to_response
@@ -14,16 +13,20 @@ logger = logging.getLogger("plain.request")
 
 
 # These middleware classes are always used by Plain.
-BUILTIN_MIDDLEWARE = [
-    "plain.internal.middleware.headers.DefaultHeadersMiddleware",
-    "plain.internal.middleware.https.HttpsRedirectMiddleware",
+BUILTIN_BEFORE_MIDDLEWARE = [
+    "plain.internal.middleware.headers.DefaultHeadersMiddleware",  # Runs after response, to set missing headers
+    "plain.internal.middleware.https.HttpsRedirectMiddleware",  # Runs before response, to redirect to HTTPS quickly
+    "plain.csrf.middleware.CsrfViewMiddleware",  # Runs before and after get_response...
+]
+
+BUILTIN_AFTER_MIDDLEWARE = [
+    # Want this to run first (when reversed) so the slash middleware
+    # can immediately redirect to the slash-appended path if there is one.
     "plain.internal.middleware.slash.RedirectSlashMiddleware",
-    "plain.csrf.middleware.CsrfViewMiddleware",
 ]
 
 
 class BaseHandler:
-    _view_middleware = None
     _middleware_chain = None
 
     def load_middleware(self):
@@ -32,12 +35,12 @@ class BaseHandler:
 
         Must be called after the environment is fixed (see __call__ in subclasses).
         """
-        self._view_middleware = []
-
         get_response = self._get_response
         handler = convert_exception_to_response(get_response)
 
-        middlewares = reversed(BUILTIN_MIDDLEWARE + settings.MIDDLEWARE)
+        middlewares = reversed(
+            BUILTIN_BEFORE_MIDDLEWARE + settings.MIDDLEWARE + BUILTIN_AFTER_MIDDLEWARE
+        )
 
         for middleware_path in middlewares:
             middleware = import_string(middleware_path)
@@ -45,13 +48,7 @@ class BaseHandler:
 
             if mw_instance is None:
                 raise ImproperlyConfigured(
-                    "Middleware factory %s returned None." % middleware_path
-                )
-
-            if hasattr(mw_instance, "process_view"):
-                self._view_middleware.insert(
-                    0,
-                    mw_instance.process_view,
+                    f"Middleware factory {middleware_path} returned None."
                 )
 
             handler = convert_exception_to_response(mw_instance)
@@ -63,7 +60,6 @@ class BaseHandler:
     def get_response(self, request):
         """Return a Response object for the given HttpRequest."""
         # Setup default url resolver for this thread
-        set_urlconf(settings.ROOT_URLCONF)
         response = self._middleware_chain(request)
         response._resource_closers.append(request.close)
         if response.status_code >= 400:
@@ -82,37 +78,23 @@ class BaseHandler:
         template_response middleware. This method is everything that happens
         inside the request/response middleware.
         """
-        response = None
-        callback, callback_args, callback_kwargs = self.resolve_request(request)
+        resolver_match = self.resolve_request(request)
 
-        # Apply view middleware
-        for middleware_method in self._view_middleware:
-            response = middleware_method(
-                request, callback, callback_args, callback_kwargs
-            )
-            if response:
-                break
-
-        if response is None:
-            response = callback(request, *callback_args, **callback_kwargs)
+        response = resolver_match.func(
+            request, *resolver_match.args, **resolver_match.kwargs
+        )
 
         # Complain if the view returned None (a common error).
-        self.check_response(response, callback)
+        self.check_response(response, resolver_match.func)
 
         return response
 
     def resolve_request(self, request):
         """
-        Retrieve/set the urlconf for the request. Return the view resolved,
+        Retrieve/set the urlrouter for the request. Return the view resolved,
         with its args and kwargs.
         """
-        # Work out the resolver.
-        if hasattr(request, "urlconf"):
-            urlconf = request.urlconf
-            set_urlconf(urlconf)
-            resolver = get_resolver(urlconf)
-        else:
-            resolver = get_resolver()
+        resolver = get_resolver()
         # Resolve the view, and assign the match object back to the request.
         resolver_match = resolver.resolve(request.path_info)
         request.resolver_match = resolver_match
@@ -126,20 +108,8 @@ class BaseHandler:
             if isinstance(callback, types.FunctionType):  # FBV
                 name = f"The view {callback.__module__}.{callback.__name__}"
             else:  # CBV
-                name = "The view {}.{}.__call__".format(
-                    callback.__module__,
-                    callback.__class__.__name__,
-                )
+                name = f"The view {callback.__module__}.{callback.__class__.__name__}.__call__"
         if response is None:
             raise ValueError(
-                "%s didn't return a Response object. It returned None "
-                "instead." % name
+                f"{name} didn't return a Response object. It returned None instead."
             )
-
-
-def reset_urlconf(sender, **kwargs):
-    """Reset the URLconf after each request is finished."""
-    set_urlconf(None)
-
-
-request_finished.connect(reset_urlconf)
